@@ -24,7 +24,7 @@
   @author Samuel Mimram, Julien Cristau, David Baelde
   *)
 
-(* $Id: vorbis.ml 2593 2006-07-07 08:03:09Z dbaelde $ *)
+(* $Id: vorbis.ml 2907 2007-03-02 13:32:11Z metamorph68 $ *)
 
 (* TODO: there are too many assert(). Some of them should be handled
  more gracefully *)
@@ -130,7 +130,7 @@ type info = {
   bitrate_minimum : int option;
   blocksize_0 : int;
   blocksize_1 : int;
-  duration : int;
+  duration : float;
 }
 
 external get_dec_file_bitstream : dec_file -> int = "ocaml_vorbis_get_dec_file_bitstream"
@@ -359,10 +359,10 @@ let get_info ?(duration=true) filename =
         blocksize_1 = blocksize_1;
         duration =
           if duration then
-            Int64.to_int
-              (Int64.div (samples filename) (Int64.of_int sample_rate))
+            (Int64.to_float (samples filename)) /.
+            (float_of_int sample_rate)
           else
-            -1
+            -1.
       }
 
 let file_size filename =
@@ -627,8 +627,8 @@ struct
         bitrate_minimum = header.bitmin;
         blocksize_0 = header.block0;
         blocksize_1 = header.block1;
-        duration = Int64.to_int(
-          Int64.div samples (Int64.of_int header.sample_rate));
+        duration =
+          (Int64.to_float samples) /. (float_of_int header.sample_rate);
       }
 
   let get_info filename =
