@@ -25,7 +25,7 @@
   * @author Samuel Mimram, Julien Cristau, David Baelde
   *)
 
-(* $Id: vorbis.mli 2593 2006-07-07 08:03:09Z dbaelde $ *)
+(* $Id: vorbis.mli 2907 2007-03-02 13:32:11Z metamorph68 $ *)
 
 (** {1 Exceptions} *)
 
@@ -203,7 +203,7 @@ type info = {
   bitrate_minimum : int option; (** minimal bitrate in kbit/s *)
   blocksize_0 : int;
   blocksize_1 : int;
-  duration : int; (** duration of the file in seconds *)
+  duration : float; (** duration of the file in seconds *)
 }
 
 (** Get the index of the sequential logical bitstream currently being decoded
