@@ -27,7 +27,7 @@
  * @author Samuel Mimram, Julien Cristau, David Baelde
  */
 
-/* $Id: vorbis_stubs.c 2693 2006-10-09 17:04:50Z smimram $ */
+/* $Id: vorbis_stubs.c 2907 2007-03-02 13:32:11Z metamorph68 $ */
 
 /*
 TODO:
@@ -155,7 +155,7 @@ inline static char* string_of_option(value v)
 
 inline static char* strncat_safe(char* dest, char* src, int pos, int len)
 {
-  dest = (char*)realloc((void*)dest, sizeof(char)*(pos+len));
+  dest = (char*)realloc((void*)dest, pos+len);
   memcpy(dest+pos, src, len);
   return dest;
 }
@@ -216,6 +216,7 @@ static value ocaml_vorbis_get_header_frame(value encoder, value title, value art
 
     build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(comment), string_of_option(date), string_of_option(genre));
     comments.vendor = malloc(strlen("ocaml-vorbis") + 1);
+    // TODO when is comments.vendor freed ?
     strcpy(comments.vendor, "ocaml-vorbis");
 
     /* Build the packets */
@@ -280,7 +281,8 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
     caml_raise_constant(*caml_named_value("vorbis_exn_invalid_samplesize"));
 
   /* We need a pointer to a myvorbis_encoder_t which won't move since vorbis seems to want to update some of the fields */
-  encoder = caml_alloc_custom(&encoder_ops, sizeof(myvorbis_encoder_t*), 0, 1);
+  encoder = caml_alloc_custom(&encoder_ops, sizeof(myvorbis_encoder_t*),
+                              sizeof(myvorbis_encoder_t), 1000000);
   ve = malloc(sizeof(myvorbis_encoder_t));
   Encoder_val(encoder) = ve;
 
@@ -370,17 +372,25 @@ CAMLprim value ocaml_vorbis_encode_buffer(value v_e, value _buf, value _ofs, val
   myvorbis_encoder_t *ve = Encoder_val(v_e);
   int len = Int_val(_len);
   int ofs = Int_val(_ofs);
-  char* in = (char*)malloc(len*sizeof(char));
+  char* in;
 
   if (ofs + len > caml_string_length(_buf))
     caml_invalid_argument("buffer too small (Vorbis.encode_buffer)");
-  memcpy(in, String_val(_buf)+ofs, len*sizeof(char));
+  in = (char*)malloc(len);
+  memcpy(in, String_val(_buf)+ofs, len);
 
   /* Ask vorbis the adress where to put the new data.
    * TODO some data can be lost when downsampling: with input at 22050Hz
    * and output at 44100Hz, one sample is lost if the input contains an odd
    * number of samples. It shouldn't hurt too much. */
   int realsamples = (len * 8 * ve->samplefreq) / (ve->in_samplesize * ve->in_channels * ve->in_samplefreq);
+  /* TODO BUG XXX WARNING etc.
+   * When doing downsampling (namely output.icecast(bitrate=32,freq=22050) in
+   * liquidsoap) it turns out that vorbis consumes more and more memory,
+   * until exhaustion. At some point vorbis_analysis_buffer allocates two 50MB
+   * buffers for PCM data! A weird thing to note is that the memory consumption
+   * seems to be more or less stable for 8-10 hours, and suddenly explodes
+   * at some point. To be continued... */
   float** buffer = vorbis_analysis_buffer(&ve->vd, realsamples);
   int i, j, c;
   int downsample = ve->in_samplefreq / ve->samplefreq ;
@@ -497,6 +507,7 @@ typedef struct myvorbis__dec_file_t
 
 static void finalize_dec_file(value df)
 {
+  // TODO check that close has been done, otherwise df->ovh leaks
   free(Decfile_val(df));
 }
 
@@ -516,10 +527,11 @@ CAMLprim value ocaml_vorbis_open_dec_file(value fd, value params)
   value block;
   int ret;
   FILE *in = NULL;
-  myvorbis_dec_file_t *df = malloc(sizeof(myvorbis_dec_file_t));
+  myvorbis_dec_file_t *df;
 
   if(!(in = fdopen(Int_val(fd), "rb")))
     caml_raise_constant(*caml_named_value("vorbis_exn_could_not_open_file"));
+  df = malloc(sizeof(myvorbis_dec_file_t));
   df->ovf = (OggVorbis_File*)malloc(sizeof(OggVorbis_File));
   df->sample_size = Int_val(Field(params, 0));
   df->big_endian = Bool_val(Field(params, 1));
@@ -538,10 +550,13 @@ CAMLprim value ocaml_vorbis_open_dec_file(value fd, value params)
   {
     fclose(in);
     free(df->ovf);
+    free(df);
     raise_err(ret);
   }
 
-  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*), 0, 1);
+  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*),
+                            sizeof(myvorbis_dec_file_t)+sizeof(OggVorbis_File),
+                            1000000);
   Decfile_val(block) = df;
 
   CAMLreturn(block);
@@ -604,7 +619,9 @@ CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, va
   int ret = 0;
   myvorbis_dec_file_t *df;
 
-  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*), 0, 1);
+  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*),
+                            sizeof(myvorbis_dec_file_t)+sizeof(OggVorbis_File),
+                            1000000);
   df = malloc(sizeof(myvorbis_dec_file_t));
   Decfile_val(block) = df;
 
@@ -643,10 +660,16 @@ CAMLprim value ocaml_vorbis_decode(value d_f, value buf_, value ofs_, value len_
   int ret = 0;
   int ofs = Int_val(ofs_);
   int len = Int_val(len_);
-  char* buf = malloc(len);
+  char* buf;
 
+  if (df->ovf==NULL)
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_parameters"));
   if (ofs + len > caml_string_length(buf_))
     caml_raise_constant(*caml_named_value("vorbis_exn_invalid_parameters"));
+  // TODO this buffer could be allocated once when creating the decoder
+  //      and reused for every decoding pass. This might be useful to reduce
+  //      load or memory fragmentation if needed.
+  buf = malloc(len);
 
   // We have to make sure that when a callback is called, the ocaml master lock
   // has been released.  Callbacks are responsible for taking it back if they
@@ -655,15 +678,10 @@ CAMLprim value ocaml_vorbis_decode(value d_f, value buf_, value ofs_, value len_
   ret = ov_read(df->ovf, buf, len, df->big_endian, df->sample_size / 8, df->sign, &df->bitstream);
   caml_leave_blocking_section();
 
-  if (ret == 0)
+  if (ret <= 0)
   {
     free(buf);
-    caml_raise_end_of_file();
-  }
-  if (ret < 0)
-  {
-    free(buf);
-    raise_err(ret);
+    ret?raise_err(ret):caml_raise_end_of_file();
   }
   memcpy(String_val(buf_) + ofs, buf, len);
   free(buf);
@@ -683,11 +701,15 @@ CAMLprim value ocaml_vorbis_close_dec_file(value d_f)
     caml_remove_global_root(&df->seek_func);
     caml_remove_global_root(&df->close_func);
     caml_remove_global_root(&df->tell_func);
+    df->read_func=0;
+  }
+  if (df->ovf) {
+    caml_enter_blocking_section();
+    ov_clear(df->ovf);
+    caml_leave_blocking_section();
+    free(df->ovf);
+    df->ovf=NULL;
   }
-  caml_enter_blocking_section();
-  ov_clear(df->ovf);
-  caml_leave_blocking_section();
-  free(df->ovf);
 
   CAMLreturn(Val_unit);
 }
